/*
 * Lua5.c
 *
 * Lua for PalmOS main PRC File.
 *
 */
 
#include <PalmOS.h>
#include <PalmOSGlue.h>

/* MathLib headers */
#include "MathLib.h"

/* Lua headers */
#include "lua.h"
#include "lauxlib.h" 
#include "lualib.h" 
#include "lua_binding_lib.h"

/* Program headers */
#include "Lua5.h"
#include "Lua5_Rsc.h"
#include "FrmSourceCode.h"


/* Extra lib headers */
#include "PalmFld.h"


/*********************************************************************
 * Entry Points
 *********************************************************************/

/*********************************************************************
 * Global variables
 *********************************************************************/
lua_State 	* gpLua = NULL;
char 		* pScript = NULL;


/*********************************************************************
 * Internal Constants
 *********************************************************************/

/* Define the minimum OS version we support */
#define ourMinVersion    sysMakeROMVersion(3,5,0,sysROMStageDevelopment,0)
#define kPalmOS20Version sysMakeROMVersion(2,0,0,sysROMStageDevelopment,0)



/*********************************************************************
 * Internal Functions
 *********************************************************************/

/*
 * 
 *   LoadLuaResource
 * 
 *   Copy LuaScript from resource to a pointer (pScript)
 *
 */
static void LoadLuaResource(void) {
	MemHandle hfrom;
	void * pfrom;
	UInt32 size;
	
	/* Get script from Resource */
	hfrom = DmGetResource('Lua5', LuaScriptResource);
	pfrom = (char *) MemHandleLock(hfrom);
	size = MemHandleSize(hfrom);
	
	/* New ptr & Copy script */
	pScript = MemPtrNew(size+1); /* +1 = '\0' at the end. */
	MemMove(pScript, pfrom, size);
	pScript[size] = '\0';
	
	/* Unlock all handles... */
	MemHandleUnlock(hfrom);
	DmReleaseResource(hfrom);
}


/*
 * 
 *   CloseLuaResource
 * 
 *   Close (free) pScript, if loaded
 *
 */
static void CloseLuaResource(void) {
	if (pScript != NULL) {
		MemPtrFree(pScript);
	}
}


/*
 * 
 *   LuaRun
 * 
 *   Run pScript
 *
 */
static void LuaRun(const char * pszScript) {
	/* Open lua */
	gpLua = lua_open();
	luaopen_base(gpLua);
	lua_tablibopen(gpLua);
	lua_strlibopen(gpLua);
	lua_iolibopen(gpLua);
	lua_mathlibopen (gpLua);
	//lua_dblibopen (gpLua);
	
	luaopen_lua5util(gpLua);
	
	/* Execute script */
	lua_dostring (gpLua, pszScript);
					
	/* Close Lua */
	if (gpLua != NULL) lua_close(gpLua);
}


/*
 * FUNCTION: MainFormHandleEvent
 *
 * DESCRIPTION:
 *
 * This routine is the event handler for the "MainForm" of this 
 * application.
 *
 * PARAMETERS:
 *
 * eventP
 *     a pointer to an EventType structure
 *
 * RETURNED:
 *     true if the event was handled and should not be passed to
 *     FrmHandleEvent
 */
static Boolean MainFormHandleEvent(EventType * eventP)
{
	Boolean handled = false;

	switch (eventP->eType)
	{
		case frmOpenEvent:
			FrmDrawForm(FrmGetActiveForm());
			handled = true;
			break;
            
		case ctlSelectEvent:
		{
			switch(eventP->data.ctlSelect.controlID) {
			
				case MainRunButton:
					LuaRun(pScript);
					break;
				
				case MainSourceButton:
					FrmPopupForm(FrmSourceCode);
					break;
					
			}
			break;
		}
	}
    
	return handled;
}

/*
 * FUNCTION: AppHandleEvent
 *
 * DESCRIPTION: 
 *
 * This routine loads form resources and set the event handler for
 * the form loaded.
 *
 * PARAMETERS:
 *
 * event
 *     a pointer to an EventType structure
 *
 * RETURNED:
 *     true if the event was handled and should not be passed
 *     to a higher level handler.
 */

static Boolean AppHandleEvent(EventType * eventP)
{
	UInt16 formId;
	FormType * frmP;

	if (eventP->eType == frmLoadEvent)
	{
		/* Load the form resource. */
		formId = eventP->data.frmLoad.formID;
		frmP = FrmInitForm(formId);
		FrmSetActiveForm(frmP);

		/* 
		 * Set the event handler for the form.  The handler of the
		 * currently active form is called by FrmHandleEvent each
		 * time is receives an event. 
		 */
		switch (formId)
		{
			case MainForm:
				FrmSetEventHandler(frmP, MainFormHandleEvent);
				break;
				
			case FrmSourceCode:
				FrmSetEventHandler(frmP, FrmSourceCodeHandleEvent);
				break;

		}
		return true;
	}

	return false;
}

/*
 * FUNCTION: AppEventLoop
 *
 * DESCRIPTION: This routine is the event loop for the application.
 */

static void AppEventLoop(void)
{
	UInt16 error;
	EventType event;

	do 
	{
		/* change timeout if you need periodic nilEvents */
		EvtGetEvent(&event, evtWaitForever);

		if (! SysHandleEvent(&event))
		{
			if (! MenuHandleEvent(0, &event, &error))
			{
				if (! AppHandleEvent(&event))
				{
					FrmDispatchEvent(&event);
				}
			}
		}
	} while (event.eType != appStopEvent);
}

/*
 * FUNCTION: AppStart
 *
 * DESCRIPTION:  Get the current application's preferences.
 *
 * RETURNED:
 *     errNone - if nothing went wrong
 */
static Err AppStart(void)
{

	/* load the MathLib library, save reference in MathLibRef */
	{
		MathLibRef = sysInvalidRefNum;

		if (errNone != SysLibFind(MathLibName, &MathLibRef))
		{
			
			/* Try to open native */
			SysLibLoad(LibType, MathLibCreator, &MathLibRef);
			
			/* Couldn't open PDA native, will create our own. */
			if (MathLibRef == sysInvalidRefNum) {
				
				MemHandle hfrom;
				void * pfrom;
				hfrom = DmGetResource('Math', MathLibResource);
				pfrom = (char *) MemHandleLock(hfrom);
				
				DmCreateDatabaseFromImage(pfrom);
			
				MemHandleUnlock(hfrom);
				DmReleaseResource(hfrom);
				hfrom = NULL;
				pfrom = NULL;
			
				SysLibLoad(LibType, MathLibCreator, &MathLibRef);
			}
			
		}
		
		if (MathLibRef != sysInvalidRefNum)
		{
			if (mlErrNone != MathLibOpen(MathLibRef, MathLibVersion))
			{
				/* if error loading library, reset reference */
				MathLibRef = sysInvalidRefNum;
			}
		}
	}
	

	// JSM.
	LoadLuaResource(); /* Loads the handle and pointer to the LuaCodeResource (Lua5). */


	return errNone;
}

/*
 * FUNCTION: AppStop
 *
 * DESCRIPTION: Save the current state of the application.
 */
static void AppStop(void)
{
        
	/* Close all the open forms. */
	FrmCloseAllForms();
	
	/* unload the MathLib library */
	if (MathLibRef != sysInvalidRefNum)
	{
		UInt16 useCount;

		MathLibClose(MathLibRef, &useCount);
		if (useCount == 0)
		{
			SysLibRemove(MathLibRef);
		}

		MathLibRef = sysInvalidRefNum;
	}
	
	CloseLuaResource();

}

/*
 * FUNCTION: RomVersionCompatible
 *
 * DESCRIPTION: 
 *
 * This routine checks that a ROM version is meet your minimum 
 * requirement.
 *
 * PARAMETERS:
 *
 * requiredVersion
 *     minimum rom version required
 *     (see sysFtrNumROMVersion in SystemMgr.h for format)
 *
 * launchFlags
 *     flags that indicate if the application UI is initialized
 *     These flags are one of the parameters to your app's PilotMain
 *
 * RETURNED:
 *     error code or zero if ROM version is compatible
 */

static Err RomVersionCompatible(UInt32 requiredVersion, UInt16 launchFlags)
{
	UInt32 romVersion;

	/* See if we're on in minimum required version of the ROM or later. */
	FtrGet(sysFtrCreator, sysFtrNumROMVersion, &romVersion);
	if (romVersion < requiredVersion)
	{
		if ((launchFlags & 
			(sysAppLaunchFlagNewGlobals | sysAppLaunchFlagUIApp)) ==
			(sysAppLaunchFlagNewGlobals | sysAppLaunchFlagUIApp))
		{
			FrmAlert (RomIncompatibleAlert);

			/* Palm OS versions before 2.0 will continuously relaunch this
			 * app unless we switch to another safe one. */
			if (romVersion < kPalmOS20Version)
			{
				AppLaunchWithCommand(
					sysFileCDefaultApp, 
					sysAppLaunchCmdNormalLaunch, NULL);
			}
		}

		return sysErrRomIncompatible;
	}

	return errNone;
}

/*
 * FUNCTION: PilotMain
 *
 * DESCRIPTION: This is the main entry point for the application.
 * 
 * PARAMETERS:
 *
 * cmd
 *     word value specifying the launch code. 
 *
 * cmdPB
 *     pointer to a structure that is associated with the launch code
 *
 * launchFlags
 *     word value providing extra information about the launch.
 *
 * RETURNED:
 *     Result of launch, errNone if all went OK
 */

UInt32 PilotMain(UInt16 cmd, MemPtr cmdPBP, UInt16 launchFlags)
{
	Err error;

	error = RomVersionCompatible (ourMinVersion, launchFlags);
	if (error) return (error);

	switch (cmd)
	{
		case sysAppLaunchCmdNormalLaunch:
			error = AppStart();
			if (error) 
				return error;

			/* 
			 * start application by opening the main form
			 * and then entering the main event loop 
			 */
			FrmGotoForm(MainForm);
			AppEventLoop();

			AppStop();
			break;
	}

	return errNone;
}
